@@ -8,7 +8,8 @@
  work, as well as functions which operate on them.
 """
 from types import FunctionType as Function
-from typing import Any, Dict, List, Type, TypeVar, Union
+from typing import Any, Callable, Dict, List, Type, TypeVar, Union, cast
+import warnings
 
 DataClass = Any  # type hint for variables that should be data class instances
 
@@ -38,6 +39,24 @@ class Hashed(metaclass=Hint):
     """Marks that a field should be included in the generated __hash__."""
 
 
+class Factory:
+    """This class takes a zero-argument callable. When a Factory instance is set as the default value of a field, this
+    callable is executed and the instance variable set to the result."""
+    Produces = TypeVar('Produces')
+
+    def __init__(self, producer: Callable[[], Produces]):
+        """The generated __init__ checks for the existence of a `copy` method to determine whether a default value
+        should be copied upon class instantiation. This is because the built-in mutable collections have a method like
+        this. This class (ab)uses this behaviour to elegantly implement the factory."""
+        self.copy = producer
+
+
+def factory(producer: Callable[[], Factory.Produces]) -> Factory.Produces:
+    """Takes a zero-argument callable and creates a Factory that executes this callable to generate a default value for
+    the field at class initialisation time. Casts the resulting Factory to keep mypy happy."""
+    return cast(Factory.Produces, Factory(producer))
+
+
 class DataClassMeta(type):
     """The metaclass that implements data class behaviour."""
     DEFAULT_OPTIONS = dict(init=True, repr=True, eq=True, iter=False, frozen=False, kwargs=False, slots=False,
@@ -96,7 +115,9 @@ class DataClassMeta(type):
             del dict_['__slots__']
 
         if options['init'] and all_annotations:  # only generate __init__ if there are fields to set
-            if post_init and '__init__' in dict_:
+            if post_init and is_user_func(dict_.get('__init__')):
+                warnings.warn('Defining post-initialisation logic in __init__ is deprecated and will be removed in a'
+                              ' forthcoming version. Please rename your method to __post_init__.', DeprecationWarning)
                 dict_['__post_init__'] = dict_.pop('__init__')
             dict_['__init__'] = generate_init(all_annotations, all_defaults, post_init,
                                               options['kwargs'], options['frozen'])
